import { CanvasStore, InstancesStore } from '@angular-three/core';
import { Directive, Input, NgZone, SkipSelf } from '@angular/core';
import { Color } from 'three';
export class ThreeMaterial {
    constructor(ngZone, instancesStore, canvasStore) {
        this.ngZone = ngZone;
        this.instancesStore = instancesStore;
        this.canvasStore = canvasStore;
    }
    set parameters(v) {
        if (v && this.material) {
            this.ngZone.runOutsideAngular(() => {
                this.convertColorToLinear(v);
                this.material.setValues(v);
                this.material.needsUpdate = true;
            });
        }
        this._parameters = v;
    }
    get parameters() {
        return this._parameters;
    }
    get material() {
        if (this._material == null) {
            this._material = new this.materialType(this.parameters);
            this.instancesStore.saveMaterial({
                id: this.ngtId,
                material: this._material,
            });
        }
        return this._material;
    }
    convertColorToLinear(parameters) {
        if ('color' in parameters) {
            const colorParams = parameters['color'];
            parameters['color'] = Array.isArray(colorParams)
                ? new Color(...colorParams)
                : new Color(colorParams);
            if (!this.canvasStore.getImperativeState().isLinear) {
                parameters['color'].convertSRGBToLinear();
            }
        }
    }
}
ThreeMaterial.decorators = [
    { type: Directive }
];
ThreeMaterial.ctorParameters = () => [
    { type: NgZone },
    { type: InstancesStore, decorators: [{ type: SkipSelf }] },
    { type: CanvasStore, decorators: [{ type: SkipSelf }] }
];
ThreeMaterial.propDecorators = {
    ngtId: [{ type: Input }],
    parameters: [{ type: Input }]
};
//# sourceMappingURL=material.abstract.js.map