import { ThreeBufferGeometry } from '@angular-three/core/geometries';
import { Directive, NgZone, Optional, Input, NgModule } from '@angular/core';
import { BufferAttribute, InstancedBufferAttribute } from 'three';
import { CommonModule } from '@angular/common';

class ThreeAttribute {
    constructor(ngZone, geometryDirective) {
        this.ngZone = ngZone;
        this.geometryDirective = geometryDirective;
        this._extraArgs = [];
    }
    set extraArgs(v) {
        this._extraArgs = v;
    }
    ngOnChanges() {
        this.ngZone.runOutsideAngular(() => {
            if (this.attribute) {
                this.attribute.needsUpdate = true;
            }
        });
    }
    ngOnInit() {
        this.ngZone.runOutsideAngular(() => {
            this._attribute = new this.attributeType(...this._extraArgs);
            if (this.geometryDirective && this.attach) {
                this.geometryDirective.bufferGeometry.setAttribute(this.attach, this.attribute);
            }
        });
    }
    ngOnDestroy() {
        this.ngZone.runOutsideAngular(() => {
            if (this.geometryDirective && this.attach) {
                this.geometryDirective.bufferGeometry.deleteAttribute(this.attach);
            }
        });
    }
    get attribute() {
        return this._attribute;
    }
}
ThreeAttribute.decorators = [
    { type: Directive }
];
ThreeAttribute.ctorParameters = () => [
    { type: NgZone },
    { type: ThreeBufferGeometry, decorators: [{ type: Optional }] }
];
ThreeAttribute.propDecorators = {
    attach: [{ type: Input }]
};

class BufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = BufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
BufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-bufferAttribute',
                exportAs: 'ngtBufferAttribute',
                providers: [
                    { provide: ThreeAttribute, useExisting: BufferAttributeDirective },
                ],
            },] }
];
BufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

class ThreeBufferAttributeModule {
}
ThreeBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    BufferAttributeDirective
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    BufferAttributeDirective
                ]
            },] }
];

class InstancedBufferAttributeDirective extends ThreeAttribute {
    constructor() {
        super(...arguments);
        this.attributeType = InstancedBufferAttribute;
    }
    set args(v) {
        this.extraArgs = v;
    }
}
InstancedBufferAttributeDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-instancedBufferAttribute',
                exportAs: 'ngtInstancedBufferAttribute',
                providers: [
                    { provide: ThreeAttribute, useExisting: InstancedBufferAttributeDirective },
                ],
            },] }
];
InstancedBufferAttributeDirective.propDecorators = {
    args: [{ type: Input }]
};

class ThreeInstancedBufferAttributeModule {
}
ThreeInstancedBufferAttributeModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    InstancedBufferAttributeDirective
                ],
                imports: [
                    CommonModule
                ],
                exports: [
                    InstancedBufferAttributeDirective
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { BufferAttributeDirective, InstancedBufferAttributeDirective, ThreeAttribute, ThreeBufferAttributeModule, ThreeInstancedBufferAttributeModule };
//# sourceMappingURL=angular-three-core-attributes.js.map
