import { InstancesStore, CanvasStore } from '@angular-three/core';
import { Directive, NgZone, SkipSelf, Input, NgModule } from '@angular/core';
import { Color, MeshLambertMaterial, MeshNormalMaterial, MeshPhongMaterial, MeshStandardMaterial } from 'three';
import { CommonModule } from '@angular/common';

class ThreeMaterial {
    constructor(ngZone, instancesStore, canvasStore) {
        this.ngZone = ngZone;
        this.instancesStore = instancesStore;
        this.canvasStore = canvasStore;
    }
    set parameters(v) {
        if (v && this.material) {
            this.ngZone.runOutsideAngular(() => {
                this.convertColorToLinear(v);
                this.material.setValues(v);
                this.material.needsUpdate = true;
            });
        }
        this._parameters = v;
    }
    get parameters() {
        return this._parameters;
    }
    get material() {
        if (this._material == null) {
            this._material = new this.materialType(this.parameters);
            this.instancesStore.saveMaterial({
                id: this.ngtId,
                material: this._material,
            });
        }
        return this._material;
    }
    convertColorToLinear(parameters) {
        if ('color' in parameters) {
            const colorParams = parameters['color'];
            parameters['color'] = Array.isArray(colorParams)
                ? new Color(...colorParams)
                : new Color(colorParams);
            if (!this.canvasStore.getImperativeState().isLinear) {
                parameters['color'].convertSRGBToLinear();
            }
        }
    }
}
ThreeMaterial.decorators = [
    { type: Directive }
];
ThreeMaterial.ctorParameters = () => [
    { type: NgZone },
    { type: InstancesStore, decorators: [{ type: SkipSelf }] },
    { type: CanvasStore, decorators: [{ type: SkipSelf }] }
];
ThreeMaterial.propDecorators = {
    ngtId: [{ type: Input }],
    parameters: [{ type: Input }]
};

class MeshLambertMaterialDirective extends ThreeMaterial {
    constructor() {
        super(...arguments);
        this.materialType = MeshLambertMaterial;
    }
}
MeshLambertMaterialDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-meshLambertMaterial',
                exportAs: 'ngtMeshLambertMaterial',
                providers: [
                    { provide: ThreeMaterial, useExisting: MeshLambertMaterialDirective },
                ],
            },] }
];

class ThreeMeshLambertMaterialModule {
}
ThreeMeshLambertMaterialModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MeshLambertMaterialDirective],
                imports: [CommonModule],
                exports: [MeshLambertMaterialDirective],
            },] }
];

class MeshNormalMaterialDirective extends ThreeMaterial {
    constructor() {
        super(...arguments);
        this.materialType = MeshNormalMaterial;
    }
}
MeshNormalMaterialDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-meshNormalMaterial',
                exportAs: 'ngtMeshNormalMaterial',
                providers: [
                    { provide: ThreeMaterial, useExisting: MeshNormalMaterialDirective },
                ],
            },] }
];

class ThreeMeshNormalMaterialModule {
}
ThreeMeshNormalMaterialModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MeshNormalMaterialDirective],
                imports: [CommonModule],
                exports: [MeshNormalMaterialDirective],
            },] }
];

class MeshPhongMaterialDirective extends ThreeMaterial {
    constructor() {
        super(...arguments);
        this.materialType = MeshPhongMaterial;
    }
}
MeshPhongMaterialDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-meshPhongMaterial',
                exportAs: 'ngtMeshPhongMaterial',
                providers: [
                    { provide: ThreeMaterial, useExisting: MeshPhongMaterialDirective },
                ],
            },] }
];

class ThreeMeshPhongMaterialModule {
}
ThreeMeshPhongMaterialModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MeshPhongMaterialDirective],
                imports: [CommonModule],
                exports: [MeshPhongMaterialDirective],
            },] }
];

class MeshStandardMaterialDirective extends ThreeMaterial {
    constructor() {
        super(...arguments);
        this.materialType = MeshStandardMaterial;
    }
}
MeshStandardMaterialDirective.decorators = [
    { type: Directive, args: [{
                selector: 'ngt-meshStandardMaterial',
                exportAs: 'ngtMeshStandardMaterial',
                providers: [
                    { provide: ThreeMaterial, useExisting: MeshStandardMaterialDirective },
                ],
            },] }
];

class ThreeMeshStandardMaterialModule {
}
ThreeMeshStandardMaterialModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MeshStandardMaterialDirective],
                imports: [CommonModule],
                exports: [MeshStandardMaterialDirective],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { MeshLambertMaterialDirective, MeshNormalMaterialDirective, MeshPhongMaterialDirective, MeshStandardMaterialDirective, ThreeMaterial, ThreeMeshLambertMaterialModule, ThreeMeshNormalMaterialModule, ThreeMeshPhongMaterialModule, ThreeMeshStandardMaterialModule };
//# sourceMappingURL=angular-three-core-materials.js.map
